@@ -57,23 +57,6 @@ class TestBoutDatasetIsXarrayDataset:
 
 
 class TestBoutDatasetMethods:
-    @pytest.mark.skip
-    def test_test_method(self, bout_xyt_example_files):
-        dataset_list = bout_xyt_example_files(None, nxpe=1, nype=1, nt=1)
-        ds = open_boutdataset(datapath=dataset_list, inputfilepath=None)
-        # ds = collect(path=path)
-        # bd = BoutAccessor(ds)
-        print(ds)
-        # ds.bout.test_method()
-        # print(ds.bout.options)
-        # print(ds.bout.metadata)
-        print(ds.isel(t=-1))
-
-        # ds.bout.set_extra_data('stored')
-        ds.bout.extra_data = "stored"
-
-        print(ds.bout.extra_data)
-
     def test_get_field_aligned(self, bout_xyt_example_files):
         dataset_list = bout_xyt_example_files(None, nxpe=3, nype=4, nt=1)
         with pytest.warns(UserWarning):
@@ -1138,6 +1121,7 @@ class TestBoutDatasetMethods:
             (
                 "n",
                 "T",
+                "S",
                 "g11",
                 "g22",
                 "g33",
@@ -1220,14 +1204,14 @@ class TestBoutDatasetMethods:
         ds["dy"].data[...] = 0.2
         ds["dz"] = 0.3
         tfunc = 1.5 * t
-        xfunc = x ** 2
-        yfunc = 10.0 * y - y ** 2
-        zfunc = 2.0 * z ** 2 - 30.0 * z
+        xfunc = x**2
+        yfunc = 10.0 * y - y**2
+        zfunc = 2.0 * z**2 - 30.0 * z
         ds["n"].data[...] = tfunc * xfunc * yfunc * zfunc
         tintegral = 48.0
         xintegral = 1000.0 / 3.0
-        yintegral = 5.0 * 22.0 ** 2 - 22.0 ** 3 / 3.0
-        zintegral = 2.0 * 36.0 ** 3 / 3.0 - 15.0 * 36.0 ** 2
+        yintegral = 5.0 * 22.0**2 - 22.0**3 / 3.0
+        zintegral = 2.0 * 36.0**3 / 3.0 - 15.0 * 36.0**2
         npt.assert_allclose(
             ds.bout.integrate_midpoints("n", dims="t"),
             (tintegral * xfunc * yfunc * zfunc).squeeze(),
@@ -1304,6 +1288,89 @@ class TestBoutDatasetMethods:
             rtol=1.4e-4,
         )
 
+        # Create and test Field2D
+        ds["S"][...] = (tfunc * xfunc * yfunc).squeeze()
+
+        # S is 'axisymmetric' so z-integral is just the length of the
+        # z-dimension
+        zintegral = 36.0
+
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims="t"),
+            (tintegral * xfunc * yfunc).squeeze(),
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims="x"),
+            (tfunc * xintegral * yfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims="y"),
+            (tfunc * xfunc * yintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims="z"),
+            (tfunc * xfunc * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["t", "x"]),
+            (tintegral * xintegral * yfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["t", "y"]),
+            (tintegral * xfunc * yintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["t", "z"]),
+            (tintegral * xfunc * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["x", "y"]),
+            (tfunc * xintegral * yintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["x", "z"]),
+            (tfunc * xintegral * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["y", "z"]),
+            (tfunc * xfunc * yintegral * zintegral).squeeze(),
+            rtol=1.2e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["t", "x", "y"]),
+            (tintegral * xintegral * yintegral),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["t", "x", "z"]),
+            (tintegral * xintegral * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["t", "y", "z"]),
+            (tintegral * xfunc * yintegral * zintegral).squeeze(),
+            rtol=1.2e-4,
+        )
+        # default dims
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S"),
+            (tfunc * xintegral * yintegral * zintegral).squeeze(),
+            rtol=1.4e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("S", dims=["t", "x", "y", "z"]),
+            (tintegral * xintegral * yintegral * zintegral),
+            rtol=1.4e-4,
+        )
+
     @pytest.mark.parametrize(
         "location", ["CELL_CENTRE", "CELL_XLOW", "CELL_YLOW", "CELL_ZLOW"]
     )
@@ -1362,28 +1429,28 @@ class TestBoutDatasetMethods:
         # Default is to integrate over all spatial dimensions
         npt.assert_allclose(
             ds.bout.integrate_midpoints("n"),
-            2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            2.0 * np.pi * R * np.pi * (router**2 - rinner**2),
             rtol=1.0e-5,
             atol=0.0,
         )
         # Pass all spatial dims explicitly
         npt.assert_allclose(
             ds.bout.integrate_midpoints("n", dims=["x", "theta", "zeta"]),
-            2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            2.0 * np.pi * R * np.pi * (router**2 - rinner**2),
             rtol=1.0e-5,
             atol=0.0,
         )
         # Integrate in time too
         npt.assert_allclose(
             ds.bout.integrate_midpoints("n", dims=["t", "x", "theta", "zeta"]),
-            T_total * 2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            T_total * 2.0 * np.pi * R * np.pi * (router**2 - rinner**2),
             rtol=1.0e-5,
             atol=0.0,
         )
         # Integrate in time using dims=...
         npt.assert_allclose(
             ds.bout.integrate_midpoints("n", dims=...),
-            T_total * 2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            T_total * 2.0 * np.pi * R * np.pi * (router**2 - rinner**2),
             rtol=1.0e-5,
             atol=0.0,
         )
@@ -1392,7 +1459,7 @@ class TestBoutDatasetMethods:
             ds.bout.integrate_midpoints(
                 "n", dims=["t", "x", "theta", "zeta"], cumulative_t=True
             ),
-            T_cumulative * 2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            T_cumulative * 2.0 * np.pi * R * np.pi * (router**2 - rinner**2),
             rtol=1.0e-5,
             atol=0.0,
         )
@@ -1429,14 +1496,14 @@ class TestBoutDatasetMethods:
         # router and circle with radius rinner, pi*(router**2 - rinner**2)
         npt.assert_allclose(
             ds.bout.integrate_midpoints("n", dims=["x", "theta"]),
-            np.pi * (router ** 2 - rinner ** 2),
+            np.pi * (router**2 - rinner**2),
             rtol=1.0e-5,
             atol=0.0,
         )
         # Integrate in time too
         npt.assert_allclose(
             ds.bout.integrate_midpoints("n", dims=["t", "x", "theta"]),
-            T_total * np.pi * (router ** 2 - rinner ** 2),
+            T_total * np.pi * (router**2 - rinner**2),
             rtol=1.0e-5,
             atol=0.0,
         )
@@ -1447,7 +1514,7 @@ class TestBoutDatasetMethods:
             ),
             T_cumulative[:, np.newaxis]
             * np.pi
-            * (router ** 2 - rinner ** 2)
+            * (router**2 - rinner**2)
             * np.ones(ds.sizes["zeta"])[np.newaxis, :],
             rtol=1.0e-5,
             atol=0.0,
@@ -1584,9 +1651,9 @@ class TestBoutDatasetMethods:
         #                  / (cos(theta/2)**2 + (1-r/R0)/(1+r/R0)*sin(theta/2)**2)**2
         #               )
         # field line length is int_{0}^{2 pi} dl
-        a = r ** 2
+        a = r**2
         c = (1 - r / R) / (1 + r / R)
-        b = q ** 2 * c
+        b = q**2 * c
 
         def func(theta):
             return np.sqrt(
@@ -1771,6 +1838,87 @@ class TestBoutDatasetMethods:
         n_unstruct = n_unstruct.isel(t=0, zeta=0)
         npt.assert_allclose(n_unstruct.values, n_check, atol=1.0e-7)
 
+    def test_interpolate_to_cartesian(self, bout_xyt_example_files):
+        dataset_list = bout_xyt_example_files(
+            None, lengths=(2, 16, 17, 18), nxpe=1, nype=1, nt=1
+        )
+        with pytest.warns(UserWarning):
+            ds = open_boutdataset(
+                datapath=dataset_list, inputfilepath=None, keep_xboundaries=False
+            )
+
+        ds["psixy"] = ds["g11"].copy(deep=True)
+        ds["Rxy"] = ds["g11"].copy(deep=True)
+        ds["Zxy"] = ds["g11"].copy(deep=True)
+
+        r = np.linspace(1.0, 2.0, ds.metadata["nx"])
+        theta = np.linspace(0.0, 2.0 * np.pi, ds.metadata["ny"])
+        R = r[:, np.newaxis] * np.cos(theta[np.newaxis, :])
+        Z = r[:, np.newaxis] * np.sin(theta[np.newaxis, :])
+        ds["Rxy"].values[:] = R
+        ds["Zxy"].values[:] = Z
+
+        ds = apply_geometry(ds, "toroidal")
+
+        ds["n"].values[:] = 1.0
+
+        nX = 30
+        nY = 30
+        nZ = 10
+        ds_cartesian = ds.bout.interpolate_to_cartesian(nX, nY, nZ)
+
+        # Check a point inside the original grid
+        npt.assert_allclose(
+            ds_cartesian["n"]
+            .isel(t=0, X=round(nX * 4 / 5), Y=nY // 2, Z=nZ // 2)
+            .item(),
+            1.0,
+            rtol=1.0e-15,
+            atol=1.0e-15,
+        )
+        # Check a point outside the original grid
+        assert np.isnan(ds_cartesian["n"].isel(t=0, X=0, Y=0, Z=0).item())
+        # Check output is float32
+        assert ds_cartesian["n"].dtype == np.float32
+
+    def test_add_cartesian_coordinates(self, bout_xyt_example_files):
+        dataset_list = bout_xyt_example_files(None, nxpe=1, nype=1, nt=1)
+        with pytest.warns(UserWarning):
+            ds = open_boutdataset(
+                datapath=dataset_list, inputfilepath=None, keep_xboundaries=False
+            )
+
+        ds["psixy"] = ds["g11"].copy(deep=True)
+        ds["Rxy"] = ds["g11"].copy(deep=True)
+        ds["Zxy"] = ds["g11"].copy(deep=True)
+
+        R0 = 3.0
+        r = np.linspace(1.0, 2.0, ds.metadata["nx"])
+        theta = np.linspace(0.0, 2.0 * np.pi, ds.metadata["ny"])
+        R = R0 + r[:, np.newaxis] * np.cos(theta[np.newaxis, :])
+        Z = r[:, np.newaxis] * np.sin(theta[np.newaxis, :])
+        ds["Rxy"].values[:] = R
+        ds["Zxy"].values[:] = Z
+
+        ds = apply_geometry(ds, "toroidal")
+
+        zeta = ds["zeta"].values
+
+        ds = ds.bout.add_cartesian_coordinates()
+
+        npt.assert_allclose(
+            ds["X_cartesian"],
+            R[:, :, np.newaxis] * np.cos(zeta[np.newaxis, np.newaxis, :]),
+        )
+        npt.assert_allclose(
+            ds["Y_cartesian"],
+            R[:, :, np.newaxis] * np.sin(zeta[np.newaxis, np.newaxis, :]),
+        )
+        npt.assert_allclose(
+            ds["Z_cartesian"],
+            Z[:, :, np.newaxis] * np.ones(ds.metadata["nz"])[np.newaxis, np.newaxis, :],
+        )
+
 
 class TestLoadInputFile:
     @pytest.mark.skip
@@ -1796,10 +1944,10 @@ class TestLoadLogFile:
 
 
 class TestSave:
-    def test_save_all(self, tmpdir_factory, bout_xyt_example_files):
+    def test_save_all(self, tmp_path_factory, bout_xyt_example_files):
         # Create data
         path = bout_xyt_example_files(
-            tmpdir_factory, nxpe=4, nype=5, nt=1, write_to_disk=True
+            tmp_path_factory, nxpe=4, nype=5, nt=1, write_to_disk=True
         )
 
         # Load it as a boutdataset
@@ -1807,7 +1955,8 @@ class TestSave:
             original = open_boutdataset(datapath=path, inputfilepath=None)
 
         # Save it to a netCDF file
-        savepath = str(Path(path).parent) + "temp_boutdata.nc"
+        savedir = tmp_path_factory.mktemp("test_save_all")
+        savepath = savedir.joinpath("temp_boutdata.nc")
         original.bout.save(savepath=savepath)
 
         # Load it again using bare xarray
@@ -1817,13 +1966,13 @@ class TestSave:
         xrt.assert_equal(original, recovered)
 
     @pytest.mark.parametrize("geometry", [None, "toroidal"])
-    def test_reload_all(self, tmpdir_factory, bout_xyt_example_files, geometry):
+    def test_reload_all(self, tmp_path_factory, bout_xyt_example_files, geometry):
         # Create data
         path = bout_xyt_example_files(
-            tmpdir_factory, nxpe=4, nype=5, nt=1, grid="grid", write_to_disk=True
+            tmp_path_factory, nxpe=4, nype=5, nt=1, grid="grid", write_to_disk=True
         )
 
-        gridpath = str(Path(path).parent) + "/grid.nc"
+        gridpath = path.parent.joinpath("grid.nc")
 
         # Load it as a boutdataset
         if geometry is None:
@@ -1843,7 +1992,8 @@ class TestSave:
             )
 
         # Save it to a netCDF file
-        savepath = str(Path(path).parent) + "/temp_boutdata.nc"
+        savedir = tmp_path_factory.mktemp("test_reload_all")
+        savepath = savedir.joinpath("temp_boutdata.nc")
         original.bout.save(savepath=savepath)
 
         # Load it again
@@ -1875,12 +2025,12 @@ class TestSave:
         "separate_vars", [False, pytest.param(True, marks=pytest.mark.long)]
     )
     def test_save_dtype(
-        self, tmpdir_factory, bout_xyt_example_files, save_dtype, separate_vars
+        self, tmp_path_factory, bout_xyt_example_files, save_dtype, separate_vars
     ):
 
         # Create data
         path = bout_xyt_example_files(
-            tmpdir_factory, nxpe=1, nype=1, nt=1, write_to_disk=True
+            tmp_path_factory, nxpe=1, nype=1, nt=1, write_to_disk=True
         )
 
         # Load it as a boutdataset
@@ -1888,7 +2038,8 @@ class TestSave:
             original = open_boutdataset(datapath=path, inputfilepath=None)
 
         # Save it to a netCDF file
-        savepath = str(Path(path).parent) + "/temp_boutdata.nc"
+        savedir = tmp_path_factory.mktemp("test_save_dtype")
+        savepath = savedir.joinpath("temp_boutdata.nc")
         original.bout.save(
             savepath=savepath, save_dtype=save_dtype, separate_vars=separate_vars
         )
@@ -1896,7 +2047,7 @@ class TestSave:
         # Load it again using bare xarray
         if separate_vars:
             for v in ["n", "T"]:
-                savepath = str(Path(path).parent) + f"/temp_boutdata_{v}.nc"
+                savepath = savedir.joinpath(f"temp_boutdata_{v}.nc")
                 recovered = open_dataset(savepath)
                 assert recovered[v].values.dtype == np.dtype(save_dtype)
         else:
@@ -1905,9 +2056,9 @@ class TestSave:
             for v in original:
                 assert recovered[v].values.dtype == np.dtype(save_dtype)
 
-    def test_save_separate_variables(self, tmpdir_factory, bout_xyt_example_files):
+    def test_save_separate_variables(self, tmp_path_factory, bout_xyt_example_files):
         path = bout_xyt_example_files(
-            tmpdir_factory, nxpe=4, nype=1, nt=1, write_to_disk=True
+            tmp_path_factory, nxpe=4, nype=1, nt=1, write_to_disk=True
         )
 
         # Load it as a boutdataset
@@ -1915,25 +2066,26 @@ class TestSave:
             original = open_boutdataset(datapath=path, inputfilepath=None)
 
         # Save it to a netCDF file
-        savepath = str(Path(path).parent) + "/temp_boutdata.nc"
+        savedir = tmp_path_factory.mktemp("test_save_separate_variables")
+        savepath = savedir.joinpath("temp_boutdata.nc")
         original.bout.save(savepath=savepath, separate_vars=True)
 
         for var in ["n", "T"]:
             # Load it again using bare xarray
-            savepath = str(Path(path).parent) + "/temp_boutdata_" + var + ".nc"
+            savepath = savedir.joinpath(f"temp_boutdata_{var}.nc")
             recovered = open_dataset(savepath)
 
             # Compare equal (not identical because attributes are changed when saving)
             xrt.assert_equal(recovered[var], original[var])
 
         # test open_boutdataset() on dataset saved with separate_vars=True
-        savepath = str(Path(path).parent) + "/temp_boutdata_*.nc"
+        savepath = savedir.joinpath("temp_boutdata_*.nc")
         recovered = open_boutdataset(savepath)
         xrt.assert_identical(original, recovered)
 
     @pytest.mark.parametrize("geometry", [None, "toroidal"])
     def test_reload_separate_variables(
-        self, tmpdir_factory, bout_xyt_example_files, geometry
+        self, tmp_path_factory, bout_xyt_example_files, geometry
     ):
         if geometry is not None:
             grid = "grid"
@@ -1941,11 +2093,11 @@ class TestSave:
             grid = None
 
         path = bout_xyt_example_files(
-            tmpdir_factory, nxpe=4, nype=1, nt=1, grid=grid, write_to_disk=True
+            tmp_path_factory, nxpe=4, nype=1, nt=1, grid=grid, write_to_disk=True
         )
 
         if grid is not None:
-            gridpath = str(Path(path).parent) + "/grid.nc"
+            gridpath = path.parent.joinpath("grid.nc")
         else:
             gridpath = None
 
@@ -1967,11 +2119,12 @@ class TestSave:
             )
 
         # Save it to a netCDF file
-        savepath = str(Path(path).parent) + "/temp_boutdata.nc"
+        savedir = tmp_path_factory.mktemp("test_reload_separate_variables")
+        savepath = savedir.joinpath("temp_boutdata.nc")
         original.bout.save(savepath=savepath, separate_vars=True)
 
         # Load it again
-        savepath = str(Path(path).parent) + "/temp_boutdata_*.nc"
+        savepath = savedir.joinpath("temp_boutdata_*.nc")
         recovered = open_boutdataset(savepath)
 
         # Compare
@@ -1979,7 +2132,7 @@ class TestSave:
 
     @pytest.mark.parametrize("geometry", [None, "toroidal"])
     def test_reload_separate_variables_time_split(
-        self, tmpdir_factory, bout_xyt_example_files, geometry
+        self, tmp_path_factory, bout_xyt_example_files, geometry
     ):
         if geometry is not None:
             grid = "grid"
@@ -1987,11 +2140,11 @@ class TestSave:
             grid = None
 
         path = bout_xyt_example_files(
-            tmpdir_factory, nxpe=4, nype=1, nt=1, grid=grid, write_to_disk=True
+            tmp_path_factory, nxpe=4, nype=1, nt=1, grid=grid, write_to_disk=True
         )
 
         if grid is not None:
-            gridpath = str(Path(path).parent) + "/grid.nc"
+            gridpath = path.parent.joinpath("grid.nc")
         else:
             gridpath = None
 
@@ -2014,17 +2167,18 @@ class TestSave:
 
         # Save it to a netCDF file
         tcoord = original.metadata.get("bout_tdim", "t")
-        savepath = str(Path(path).parent) + "/temp_boutdata_1.nc"
+        savedir = tmp_path_factory.mktemp("test_reload_separate_variables_time_split")
+        savepath = savedir.joinpath("temp_boutdata_1.nc")
         original.isel({tcoord: slice(3)}).bout.save(
             savepath=savepath, separate_vars=True
         )
-        savepath = str(Path(path).parent) + "/temp_boutdata_2.nc"
+        savepath = savedir.joinpath("temp_boutdata_2.nc")
         original.isel({tcoord: slice(3, None)}).bout.save(
             savepath=savepath, separate_vars=True
         )
 
         # Load it again
-        savepath = str(Path(path).parent) + "/temp_boutdata_*.nc"
+        savepath = savedir.joinpath("temp_boutdata_*.nc")
         recovered = open_boutdataset(savepath)
 
         # Compare
@@ -2033,16 +2187,18 @@ class TestSave:
 
 class TestSaveRestart:
     @pytest.mark.parametrize("tind", [None, pytest.param(1, marks=pytest.mark.long)])
-    def test_to_restart(self, tmpdir_factory, bout_xyt_example_files, tind):
+    def test_to_restart(self, tmp_path_factory, bout_xyt_example_files, tind):
         nxpe = 3
         nype = 2
 
+        nt = 6
+
         path = bout_xyt_example_files(
-            tmpdir_factory,
+            tmp_path_factory,
             nxpe=nxpe,
             nype=nype,
             nt=1,
-            lengths=[6, 4, 4, 7],
+            lengths=[nt, 4, 4, 7],
             guards={"x": 2, "y": 2},
             write_to_disk=True,
         )
@@ -2055,7 +2211,7 @@ class TestSaveRestart:
         ny = ds.metadata["ny"]
 
         # Save it to a netCDF file
-        savepath = Path(path).parent
+        savepath = tmp_path_factory.mktemp("test_to_restart")
         if tind is None:
             ds.bout.to_restart(savepath=savepath, nxpe=nxpe, nype=nype)
         else:
@@ -2089,30 +2245,38 @@ class TestSaveRestart:
                 # No coordinates saved in restart files, so unset them in check_ds
                 check_ds = check_ds.reset_coords()
 
+                # ignore variables that depend on the rank of the BOUT++ process - they
+                # cannot be consistent with check_ds
+                rank_dependent_vars = ["PE_XIND", "PE_YIND", "MYPE"]
                 for v in restart_ds:
                     if v in check_ds:
                         xrt.assert_equal(restart_ds[v], check_ds[v])
                     else:
                         if v == "hist_hi":
-                            assert restart_ds[v].values == -1
+                            if t >= 0:
+                                assert restart_ds[v].values == t
+                            else:
+                                assert restart_ds[v].values == nt - 1
                         elif v == "tt":
                             assert restart_ds[v].values == t_array
-                        else:
+                        elif v not in rank_dependent_vars:
                             assert restart_ds[v].values == check_ds.metadata[v]
 
-    def test_to_restart_change_npe(self, tmpdir_factory, bout_xyt_example_files):
+    def test_to_restart_change_npe(self, tmp_path_factory, bout_xyt_example_files):
         nxpe_in = 3
         nype_in = 2
 
         nxpe = 2
         nype = 4
 
+        nt = 6
+
         path = bout_xyt_example_files(
-            tmpdir_factory,
+            tmp_path_factory,
             nxpe=nxpe_in,
             nype=nype_in,
             nt=1,
-            lengths=[6, 4, 4, 7],
+            lengths=[nt, 4, 4, 7],
             guards={"x": 2, "y": 2},
             write_to_disk=True,
         )
@@ -2125,7 +2289,7 @@ class TestSaveRestart:
         ny = ds.metadata["ny"]
 
         # Save it to a netCDF file
-        savepath = Path(path).parent
+        savepath = tmp_path_factory.mktemp("test_to_restart_change_npe")
         ds.bout.to_restart(savepath=savepath, nxpe=nxpe, nype=nype)
 
         mxsub = (nx - 4) // nxpe
@@ -2151,6 +2315,9 @@ class TestSaveRestart:
                 # No coordinates saved in restart files, so unset them in check_ds
                 check_ds = check_ds.reset_coords()
 
+                # ignore variables that depend on the rank of the BOUT++ process - they
+                # cannot be consistent with check_ds
+                rank_dependent_vars = ["PE_XIND", "PE_YIND", "MYPE"]
                 for v in restart_ds:
                     if v in check_ds:
                         xrt.assert_equal(restart_ds[v], check_ds[v])
@@ -2158,15 +2325,15 @@ class TestSaveRestart:
                         if v in ["NXPE", "NYPE", "MXSUB", "MYSUB"]:
                             pass
                         elif v == "hist_hi":
-                            assert restart_ds[v].values == -1
+                            assert restart_ds[v].values == nt - 1
                         elif v == "tt":
                             assert restart_ds[v].values == t_array
-                        else:
+                        elif v not in rank_dependent_vars:
                             assert restart_ds[v].values == check_ds.metadata[v]
 
     @pytest.mark.long
     def test_to_restart_change_npe_doublenull(
-        self, tmpdir_factory, bout_xyt_example_files
+        self, tmp_path_factory, bout_xyt_example_files
     ):
         nxpe_in = 3
         nype_in = 6
@@ -2174,13 +2341,15 @@ class TestSaveRestart:
         nxpe = 1
         nype = 12
 
+        nt = 6
+
         path = bout_xyt_example_files(
-            tmpdir_factory,
+            tmp_path_factory,
             nxpe=nxpe_in,
             nype=nype_in,
             nt=1,
             guards={"x": 2, "y": 2},
-            lengths=(6, 5, 4, 7),
+            lengths=(nt, 5, 4, 7),
             topology="upper-disconnected-double-null",
             write_to_disk=True,
         )
@@ -2193,7 +2362,7 @@ class TestSaveRestart:
         ny = ds.metadata["ny"]
 
         # Save it to a netCDF file
-        savepath = Path(path).parent
+        savepath = tmp_path_factory.mktemp("test_to_restart_change_npe_doublenull")
         ds.bout.to_restart(savepath=savepath, nxpe=nxpe, nype=nype)
 
         mxsub = (nx - 4) // nxpe
@@ -2219,6 +2388,9 @@ class TestSaveRestart:
                 # No coordinates saved in restart files, so unset them in check_ds
                 check_ds = check_ds.reset_coords()
 
+                # ignore variables that depend on the rank of the BOUT++ process - they
+                # cannot be consistent with check_ds
+                rank_dependent_vars = ["PE_XIND", "PE_YIND", "MYPE"]
                 for v in restart_ds:
                     if v in check_ds:
                         xrt.assert_equal(restart_ds[v], check_ds[v])
@@ -2226,16 +2398,16 @@ class TestSaveRestart:
                         if v in ["NXPE", "NYPE", "MXSUB", "MYSUB"]:
                             pass
                         elif v == "hist_hi":
-                            assert restart_ds[v].values == -1
+                            assert restart_ds[v].values == nt - 1
                         elif v == "tt":
                             assert restart_ds[v].values == t_array
-                        else:
+                        elif v not in rank_dependent_vars:
                             assert restart_ds[v].values == check_ds.metadata[v]
 
     @pytest.mark.long
     @pytest.mark.parametrize("npes", [(2, 6), (3, 4)])
     def test_to_restart_change_npe_doublenull_expect_fail(
-        self, tmpdir_factory, bout_xyt_example_files, npes
+        self, tmp_path_factory, bout_xyt_example_files, npes
     ):
         nxpe_in = 3
         nype_in = 6
@@ -2243,7 +2415,7 @@ class TestSaveRestart:
         nxpe, nype = npes
 
         path = bout_xyt_example_files(
-            tmpdir_factory,
+            tmp_path_factory,
             nxpe=nxpe_in,
             nype=nype_in,
             nt=1,
@@ -2261,6 +2433,16 @@ class TestSaveRestart:
         ny = ds.metadata["ny"]
 
         # Save it to a netCDF file
-        savepath = Path(path).parent
+        savepath = tmp_path_factory.mktemp(
+            "test_to_restart_change_npe_doublenull_expect_fail"
+        )
         with pytest.raises(ValueError):
             ds.bout.to_restart(savepath=savepath, nxpe=nxpe, nype=nype)
+
+    def test_from_restart_to_restart(self, tmp_path):
+        datapath = Path(__file__).parent.joinpath(
+            "data", "restart", "BOUT.restart.*.nc"
+        )
+        ds = open_boutdataset(datapath, keep_xboundaries=True, keep_yboundaries=True)
+
+        ds.bout.to_restart(savepath=tmp_path, nxpe=1, nype=4)
